/*
	Action base class definition
	Copyright (c) 1994~1995 Marco Savegnago
*/

#ifndef	__TACTION_H
#define	__TACTION_H

#include 	"tdef.h"
#include    "trigitem.h"

class	TAction
{
public:
	/*
		I tipi di azioni che attualmente e' possibile far generare sono
		i seguenti:

		-	Set	 o Reset di LogicalOutput di un SYSOutputSlot
		-	Play o Stop di un CDROM in un CDROM Manager
		-	Set  delle impostazioni in un mixer/equalizzatore
	*/

	enum	IType
	{
		SYSBoard		=	0x0001,
		CDROMManager	=	0x0002,
		MixerManager	=	0x0004,
		BootCPU			=   0x0008,
		EventFlag		=	0x0010,
		CounterFlag		=	0x0020,
		Timer			=	0x0040,
		TimerManager	=	0x0080,

	} iType;

	TAction(TAction::IType iType_)
	:
		iType(iType_)
		{
		}

	virtual	~TAction() {}

	virtual	Boolean	action(word pos, Boolean set, int   value ...) = 0;
};

class	TTriggerEvent;

class	TEventItem
{
	char*			name;

public:

	TTriggerEvent*	theTrigger;
	TAction*		theTHENAction;
	word			theTHENPos;
	int				theTHENValue;
	int				theTHENSubValue;
	TAction*		theELSEAction;
	word			theELSEPos;
	int				theELSEValue;
	int				theELSESubValue;

	TBNSCollection	other;			// Collezione di CDROM

	TEventItem	(
				TTriggerEvent* 	theTrigger      = NULL,
				TAction* 		theTHENAction 	= NULL,
				word			aTHENPos 		= 0,
				int				aTHENValue 		= 0,
				TAction* 		theELSEAction 	= NULL,
				word			anELSEPos       = 0,
				int				anELSEValue		= 0
				);

	TEventItem	(
				TTriggerEvent* 	theTrigger,
				TAction* 		theTHENAction,
				word			aTHENPos,
				int				aTHENValue,
				int				aTHENSubValue,
				TAction* 		theELSEAction 	= NULL,
				word			anELSEPos       = 0,
				int				anELSEValue		= 0,
				int				anELSESubValue	= 0
				);

	TEventItem	(
				TEventItem*		parent,
				TAction* 		theTHENAction 	= NULL,
				word			aTHENPos 		= 0,
				int				aTHENValue 		= 0,
				TAction* 		theELSEAction 	= NULL,
				word			anELSEPos       = 0,
				int				anELSEValue		= 0
				);

	TEventItem	(
				TEventItem*		parent,
				TAction* 		theTHENAction,
				word			aTHENPos,
				int				aTHENValue,
				int				aTHENSubValue,
				TAction* 		theELSEAction 	= NULL,
				word			anELSEPos       = 0,
				int				anELSEValue		= 0,
				int				anELSESubValue	= 0
				);

	Boolean			resolve();
	Boolean			add(TEventItem* event);
};


class	TEventFlag
:
	public TAction,
	public TTriggerItem
{
	Boolean init;		//	Valore iniziale
	Boolean	state;
public:
	TEventFlag(Boolean initState = False);

	virtual	Boolean	action(word pos, Boolean set, int   value ...) ;
	virtual	Boolean Is() ;
	virtual	Boolean IsNot() ;
	virtual	void	Reset();

};

class	TEventCounter
:
	public TAction,
	public TTriggerItem
{
	int		init;		//	Valore iniziale
	int		state;
public:
	TEventCounter(int	initState = 0);

	virtual	Boolean	action(word pos, Boolean set, int   value ...) ;
	virtual	Boolean Is() ;
	virtual	Boolean IsNot() ;
	virtual	void	Reset();
	virtual	Boolean	IsEq(int value) ;
	int		Value();
};

inline	int		TEventCounter::Value()
{
	return	state;
}


#endif	//	__TACTION_H